package ij.gui;
import java.awt.*;
import java.awt.event.*;
import ij.*;

/** This class is a generic modal dialog box. */
public class GenericDialog extends Dialog implements ActionListener {
	static final int MAX_ITEMS = 10;
	private TextField[] numberField = new TextField[MAX_ITEMS];
	private double[] defaultValues = new double[MAX_ITEMS];
	private String[] defaultText = new String[MAX_ITEMS];
	private TextField[] stringField = new TextField[MAX_ITEMS];
	private Checkbox[] checkbox = new Checkbox[MAX_ITEMS];
	private Choice[] choice = new Choice[MAX_ITEMS];
	private Button cancel, okay;
	private Frame parent;
    private boolean wasCanceled;
    private int y;
    private int nfIndex;
    private int sfIndex;
    private int cbIndex;
    private int choiceIndex;
	private GridBagLayout grid;
	private GridBagConstraints c;
	private boolean firstNumericField = true;
	private boolean firstStringField = true;
	private boolean firstCheckbox = true;
	private boolean firstChoice = true;
	private boolean invalidNumber;
	private boolean firstPaint = true;

    public GenericDialog(String title, Frame parent) {
		super(parent, title, true);
   		this.parent = parent;
		grid = new GridBagLayout();
		c = new GridBagConstraints();
		setLayout(grid);
    }
    
	//void showFields(String id) {
	//	String s = id+": ";
	//	for (int i=0; i<MAX_ITEMS; i++)
	//		if (numberField[i]!=null)
	//			s += i+"='"+numberField[i].getText()+"' ";
	//	IJ.write(s);
	//}

   public void addNumericField(String text, double defaultValue, int digits) {
		Label label = new Label(text);
		c.gridx = 0; c.gridy = y;
		c.anchor = GridBagConstraints.EAST;
		if (firstNumericField)
			c.insets = new Insets(5, 0, 5, 0);
		else
			c.insets = new Insets(0, 0, 5, 0);
		grid.setConstraints(label, c);
		add(label);

		numberField[nfIndex] = new TextField(IJ.d2s(defaultValue, digits), 6);
		numberField[nfIndex].addActionListener(this);
		defaultValues[nfIndex] = defaultValue;
		defaultText[nfIndex] = numberField[nfIndex].getText();
		c.gridx = 1; c.gridy = y;
		c.anchor = GridBagConstraints.WEST;
		grid.setConstraints(numberField[nfIndex], c);
		numberField[nfIndex].setEditable(true);
		if (firstNumericField) numberField[nfIndex].selectAll();
		firstNumericField = false;
		add(numberField[nfIndex]);
		y++; nfIndex++;
    }
    
	public void addStringField(String text, String defaultText) {
		addStringField(text, defaultText, 8);
	}

	public void addStringField(String text, String defaultText, int columns) {
		Label label = new Label(text);
		c.gridx = 0; c.gridy = y;
		c.anchor = GridBagConstraints.EAST;
		if (firstStringField)
			c.insets = new Insets(5, 0, 5, 0);
		else
			c.insets = new Insets(0, 0, 5, 0);
		grid.setConstraints(label, c);
		add(label);

		stringField[sfIndex] = new TextField(defaultText, columns);
		stringField[sfIndex].addActionListener(this);
		c.gridx = 1; c.gridy = y;
		c.anchor = GridBagConstraints.WEST;
		grid.setConstraints(stringField[sfIndex], c);
		stringField[sfIndex].setEditable(true);
		add(stringField[sfIndex]);
		y++; sfIndex++;
    }
    
    public void addCheckbox(String text, boolean defaultValue) {
		checkbox[cbIndex] = new Checkbox(text);
		c.gridx = 0; c.gridy = y;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		if (firstCheckbox)
			c.insets = new Insets(15, 20, 0, 0);
		else
			c.insets = new Insets(0, 20, 0, 0);
		firstCheckbox = false;
		grid.setConstraints(checkbox[cbIndex], c);
		checkbox[cbIndex].setState(defaultValue);
		add(checkbox[cbIndex]);
		//ij.IJ.write("addCheckbox: "+ y+" "+cbIndex);
		y++; cbIndex++;
    }
    
    public void addChoice(String text, String[] items, String defaultItem) {
		Label label = new Label(text);
		c.gridx = 0; c.gridy = y;
		c.anchor = GridBagConstraints.EAST;
		if (firstChoice)
			c.insets = new Insets(5, 0, 5, 0);
		else
			c.insets = new Insets(0, 0, 5, 0);
		grid.setConstraints(label, c);
		add(label);
		choice[choiceIndex] = new Choice();
		for (int i=0; i<items.length; i++)
			choice[choiceIndex].addItem(items[i]);
		choice[choiceIndex].select(defaultItem);
		c.gridx = 1; c.gridy = y;
		c.anchor = GridBagConstraints.WEST;
		grid.setConstraints(choice[choiceIndex], c);
		firstChoice = false;
		add(choice[choiceIndex]);
		y++; choiceIndex++;
    }
    
    public void addMessage(String text) {
		Label label = new Label(text);
		c.gridx = 0; c.gridy = y;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(text.equals("")?0:10, 20, 0, 0);
		grid.setConstraints(label, c);
		add(label);
		y++;
    }
    
    public boolean wasCanceled() {
    	return wasCanceled;
    }
    
    public double getNextNumber() {
		Double d;
		if (numberField[nfIndex]==null)
			return -1.0;
		String theText = numberField[nfIndex].getText();
		String originalText = defaultText[nfIndex];
		double defaultValue = defaultValues[nfIndex];
		nfIndex++;
		if (theText.equals(originalText))
			return defaultValue;
		try {d = new Double(theText);}
		catch (NumberFormatException e){
			//numberField[nfIndex].setText(""+xScale);
			d = null;
		}
		if (d!=null)
			return(d.doubleValue());
		else {
			invalidNumber = true;
			return 0.0;
		}
    }
    
    public boolean invalidNumber() {
    	boolean wasInvalid = invalidNumber;
    	invalidNumber = false;
    	return wasInvalid;
    }
    
    public String getNextString() {
		Double d;
		if (stringField[sfIndex]==null)
			return "";
		else
			return stringField[sfIndex++].getText();
    }
    
    public boolean getNextBoolean() {
		if (checkbox[cbIndex]==null)
			return false;
		else
			return checkbox[cbIndex++].getState();
    }
    
    public String getNextChoice() {
		if (choice[choiceIndex]==null)
			return "";
		else
			return choice[choiceIndex++].getSelectedItem();
    }
    
    public int getNextChoiceIndex() {
		if (choice[choiceIndex]==null)
			return -1;
		else
			return choice[choiceIndex++].getSelectedIndex();
    }

    public void showDialog() {
    	if (stringField[0]!=null && numberField[0]==null)
    		stringField[0].selectAll();
		Panel buttons = new Panel();
    	buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
		cancel = new Button("Cancel");
		cancel.addActionListener(this);
		buttons.add(cancel);
		okay = new Button("  OK  ");
		okay.addActionListener(this);
		buttons.add(okay);
		c.gridx = 0; c.gridy = y;
		c.anchor = GridBagConstraints.EAST;
		c.gridwidth = 2;
		c.insets = new Insets(20, 0, 0, 0);
		grid.setConstraints(buttons, c);
		add(buttons);
		nfIndex = 0;
		sfIndex = 0;
		cbIndex = 0;
		choiceIndex = 0;
		pack();
		GUI.center(this);
		setVisible(true);
		IJ.wait(250); // work around for Sun/WinNT bug
  }
    
	public void actionPerformed(ActionEvent e) {
		wasCanceled = (e.getSource()==cancel);
		setVisible(false);
		dispose();
	}

    public Insets getInsets() {
    	return new Insets(40, 20, 20, 20);
    }

    public void paint(Graphics g) {
    	super.paint(g);
      	if (firstPaint && numberField[0]!=null)
    		numberField[0].requestFocus();
    	firstPaint = false;
    }
    
}