package ij.plugin.filter;
import ij.*;
import ij.process.*;
import ij.gui.*;

/** Draws an arrow at the location of the current straight line selection. */
public class Arrow implements PlugInFilter {
	
	ImagePlus imp;
	ImageProcessor ip;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_ALL;
	}

	public void run(ImageProcessor ip) {
		this.ip = ip;
		Roi roi = imp.getRoi();
		if (roi==null || !(roi instanceof Line)) {
			IJ.error("Straight line selection required.");
			return;
		}
		Line line = (Line)roi;
		int x1=line.x1, y1=line.y1, x2=line.x2, y2=line.y2;  
		imp.killRoi();
		drawArrow(x1,y1,x2,y2);
	}

	void drawArrow(int x1, int y1, int x2, int y2) {
		double size=12; // of arrow head in pixels
		double angle=20; // of arrow head degrees
		double dx, dy, pi, theta;
		int width = imp.getHeight();
		int height = imp.getHeight();
		ip.moveTo(x1,y1);
		ip.lineTo(x2,y2);
		y1 = width-y1;
		y2 = height-y2;
		if (Line.getWidth()>1)
			size = size + size*Line.getWidth()*0.3;
		angle = (angle/180.0)*Math.PI;
		dx = x1-x2;
		dy = y1-y2;
		if (dx==0.0) {
			if (dy>=0.0)
				theta = Math.PI/2.0;
			else
				theta = 1.5*Math.PI;
		} else {
			theta = Math.atan(dy/dx);
			if (dx<0.0) theta = theta+Math.PI;
		}
		ip.moveTo(x2, height-y2);
		ip.lineTo(x2+(int)(size*Math.cos(theta+angle)), height-(int)(y2+size*Math.sin(theta+angle)));
		ip.moveTo(x2, height-y2);
		ip.lineTo(x2+(int)(size*Math.cos(theta-angle)), height-(int)(y2+size*Math.sin(theta-angle)));
	}

}
