package ij.process;

/** 8-bit image statistics, including histogram. */
public class ByteStatistics extends ImageStatistics {

	/** Construct an ImageStatistics object from a ByteProcessor
		using the standard measurement options (area, mean,
		mode, min and max) and generating a 256 bin histogram. */
	public ByteStatistics(ImageProcessor ip) {
		this(ip, AREA+MEAN+MODE+MIN_MAX, 256);
	}

	/** Constructs a ByteStatistics object from a ByteProcessor using
		the specified measurement options and number of histogram bins.
		The number of bins is currently fixed at 256. */
	public ByteStatistics(ImageProcessor ip, int mOptions, int nBins) {
		ByteProcessor bp = (ByteProcessor)ip;
		histogram = bp.getHistogram();
		setRoiLimits(ip);
		double minThreshold = ip.getMinThreshold();
		if ((mOptions&LIMIT)==0 || minThreshold==ip.NO_THRESHOLD)
			getStatistics(0,255);
		else {
			double maxThreshold = ip.getMaxThreshold();
			getStatistics((int)minThreshold,(int)maxThreshold);
		}
		if ((mOptions&MIN_MAX)!=0)
			getMinAndMax();
		if ((mOptions&CENTROID)!=0)
			getCentroid(ip);
		if ((mOptions&CENTER_OF_MASS)!=0)
			getCenterOfMass(ip);
	}

	void getCenterOfMass(ImageProcessor ip) {
		byte[] pixels = (byte[])ip.getPixels();
		int[] mask = ip.getMask();
		int count=0, v, i, mi;
		double xsum=0.0, ysum=0.0;
		for (int y=roiY,my=0; y<(roiY+roiHeight); y++,my++) {
			i = y*width + roiX;
			mi = my*roiWidth;
			for (int x=roiX; x<(roiX+roiWidth); x++) {
				if (mask==null || mask[mi++]==ip.BLACK) {
					v = pixels[i]&255;
					count += v;
					xsum += x*v;
					ysum += y*v;
				}
				i++;
			}
		}
		xCenterOfMass = xsum/count+0.5;
		yCenterOfMass = ysum/count+0.5;
	}

}